@@ -16,7 +16,6 @@ import StringIO
 import mimetypes
 import os
 
-from urllib  import quote
 from errors  import ResponseError, NoSuchObject, \
                     InvalidObjectName, IncompleteSend, \
                     InvalidMetaName, InvalidMetaValue
@@ -75,6 +74,7 @@ class Object(object):
         self.container = container
         self.last_modified = None
         self.metadata = {}
+        self.headers = {}
         self.manifest = None
         if object_record:
             self.name = object_record['name']
@@ -212,17 +212,18 @@ class Object(object):
     @requires_name(InvalidObjectName)
     def sync_metadata(self):
         """
-        Commits the metadata to the remote storage system.
+        Commits the metadata and custom headers to the remote storage system.
 
         >>> test_object = container['paradise_lost.pdf']
         >>> test_object.metadata = {'author': 'John Milton'}
-        >>> test_object.sync_metadata()
+        >>> test_object.headers = {'content-disposition': 'foo'}
+        >>> test_objectt.sync_metadata()
 
         Object metadata can be set and retrieved through the object's
         .metadata attribute.
         """
         self._name_check()
-        if self.metadata:
+        if self.metadata or self.headers:
             headers = self._make_headers()
             headers['Content-Length'] = "0"
             response = self.container.conn.make_request(
@@ -261,7 +262,7 @@ class Object(object):
         headers['X-Auth-Token'] = self.container.conn.token
 
         path = "/%s/%s/%s" % (self.container.conn.uri.rstrip('/'), \
-                quote(self.container.name), quote(self.name))
+                unicode_quote(self.container.name), unicode_quote(self.name))
 
         # Requests are handled a little differently for writes ...
         http = self.container.conn.connection
@@ -387,7 +388,7 @@ class Object(object):
             self._etag = None
 
         headers = self._make_headers()
-        headers['Destination'] = quote("%s/%s" % (container_name, name))
+        headers['Destination'] = unicode_quote("%s/%s" % (container_name, name))
         headers['Content-Length'] = 0
         response = self.container.conn.make_request(
                    'COPY', [self.container.name, self.name], hdrs=headers, data='')
@@ -415,7 +416,7 @@ class Object(object):
             self._etag = None
 
         headers = self._make_headers()
-        headers['X-Copy-From'] = quote("%s/%s" % (container_name, name))
+        headers['X-Copy-From'] = unicode_quote("%s/%s" % (container_name, name))
         headers['Content-Length'] = 0
         response = self.container.conn.make_request(
                    'PUT', [self.container.name, self.name], hdrs=headers, data='')
@@ -609,6 +610,7 @@ class Object(object):
             if len(self.metadata[key]) > consts.meta_value_limit:
                 raise(InvalidMetaValue(self.metadata[key]))
             headers['X-Object-Meta-' + key] = self.metadata[key]
+        headers.update(self.headers)
         return headers
 
     @classmethod
@@ -623,7 +625,7 @@ class Object(object):
             buff = fobj.read(4096)
         fobj.seek(0)
         return checksum.hexdigest()
-
+    
     def public_uri(self):
         """
         Retrieve the URI for this object, if its container is public.
@@ -638,8 +640,8 @@ class Object(object):
         @rtype: str
         """
         return "%s/%s" % (self.container.public_uri().rstrip('/'),
-                quote(self.name))
-    
+            unicode_quote(self.name))
+
     def public_ssl_uri(self):
         """
         Retrieve the SSL URI for this object, if its container is public.
@@ -654,7 +656,7 @@ class Object(object):
         @rtype: str
         """
         return "%s/%s" % (self.container.public_ssl_uri().rstrip('/'),
-                quote(self.name))
+                unicode_quote(self.name))
 
     def public_streaming_uri(self):
         """
@@ -670,7 +672,7 @@ class Object(object):
         @rtype: str
         """
         return "%s/%s" % (self.container.public_streaming_uri().rstrip('/'),
-                quote(self.name))
+                unicode_quote(self.name))
 
     def purge_from_cdn(self, email=None):
         """
